clc
clear
close all

global xData yData yCalc glbRsqr QSPcoeff
global orderP orderQ
zFilename = "Opt_besselj_0_x_pade_rand_run2";
txtFile = strcat(zFilename, ".txt");
xlFile = strcat(zFilename, ".xlsx");
diary(txtFile)
gFile =  strcat(zFilename, ".jpg");
fprintf("%s\n", datetime(now,'ConvertFrom','datenum'));
format longE
sEqn = "besselj(0,x)";
fprintf("%s\n", sEqn);
fprintf("x=0:0.1:10\n")
xData= 0:0.1:10;
xData = xData';
n = length(xData);
yData = besselj(0,xData);
bestR2 = -1;
for orderP=2:6
  for orderQ=2:6
    fprintf("\norderP = %d, orderQ=%d\n\n", orderP,orderQ)
    [LbP,UbP] = makeLimits(orderP, 0.7, 1.6);
    [LbQ,UbQ] = makeLimits(orderQ, 0.7, 1.6);
    [bestX,bestFx] = randomSearchPade(@quantShammasPadePoly,LbP,UbP,LbQ,UbQ,1000000);
    
    SSE = quantShammasPadePoly(bestX);
    orderTot = orderP + orderQ;
    % calculate adjusted value of the coefficient of determination
    glbRsqr = 1 - (1 - glbRsqr)*(n-1)/(n-orderTot-1);
    if bestR2 < glbRsqr
      bestR2 = glbRsqr;
      bestP = orderP;
      bestQ = orderQ;
      bestQSPcoeff = QSPcoeff;
      zBestX = bestX;
      bestYCalc = yCalc;
    end
  end
end
fprintf("Best orderP = %d\n", bestP);
fprintf("Best orderQ = %d\n", bestQ);
fprintf("Quantum Shammas Pade Polynomial Powers\n");
zBestX
fprintf("Quantum Shammas Pade Polynomial Coefficients\n");
QSPcoeff = bestQSPcoeff'
fprintf("\nRegular Pade polynomial fit\n");
[c,r,yPoly] = padefit(xData,yData,bestP,bestQ);
% calculate adjusted value of the coefficient of determination
orderTot = bestP + bestQ;
r = 1 - (1 - r)*(n-1)/(n-orderTot-1);
fprintf("Adjusted Rsqr = %f\n", r);
figure(1)
plot(xData,yData,xData,bestYCalc,xData,yPoly);
title(sEqn)
xlabel("X")
ylabel("Y");
grid;
ax = gca;
exportgraphics(ax,gFile);

QSPpwr = zBestX;
Coeff = c';
T1 = array2table(QSPpwr);
writetable(T1,xlFile,"Sheet","Sheet1","Range","A1");
T2 = array2table(QSPcoeff);
writetable(T2,xlFile,"Sheet","Sheet1","Range","A4");
T3 = array2table(Coeff);
writetable(T3,xlFile,"Sheet","Sheet1","Range","A7");
r_sqr = [bestR2 r];
T4 = array2table(r_sqr);
writetable(T4,xlFile,"Sheet","Sheet1","Range","A10");
padeOrderP = bestP;
padeOrderQ = bestQ;
orderP = [padeOrderP];
T = array2table(orderP);
writetable(T,xlFile,"Sheet","Sheet1","Range","D10");
orderQ = [padeOrderQ];
T = array2table(orderQ);
writetable(T,xlFile,"Sheet","Sheet1","Range","E10");
format short
diary off

function [Lb,Ub] = makeLimits(order, minPwr, maxPwr)
  Lb = zeros(1,order);
  Ub = zeros(1,order);
  Lb(1) = minPwr;
  Ub(1) = maxPwr;
  for i=2:order
    j = i - 1;
    Lb(i) = j + minPwr;
    Ub(i) = j + maxPwr;
  end
end

