clc
clear
close all

global xData yData yCalc glbRsqr QSPcoeff
global orderP orderQ
zFilename = "Right_GaussBell_x_pade_sobol_random";
txtFile = strcat(zFilename, ".txt");
xlFile = strcat(zFilename, ".xlsx");
diary(txtFile)
gFile =  strcat(zFilename, ".jpg");
fprintf("%s\n", datetime(now,'ConvertFrom','datenum'));
format longE
sEqn = "exp(-x^2/2)/sqrt(2*pi)";
fprintf(sEqn);
fprintf("x=0:0.1:3\n")
xData= 0:0.1:3;
xData = xData';
n = length(xData);
yData = exp(-xData.^2/2)/sqrt(2*pi);
bestR2 = -1;
for orderP=2:6
  for orderQ=2:6
    fprintf("\norderP = %d, orderQ = %d\n\n", orderP,orderQ)
    [LbP,UbP] = makeLimits(orderP, 0.5, 0.9, 0.1);
    [LbQ,UbQ] = makeLimits(orderQ, 0.5, 0.9, 0.1);
    [bestX,bestFx] = sobolRandomSearchPade(@quantShammasPadePoly,LbP,UbP,LbQ,UbQ,2000000);
        
    SSE = quantShammasPadePoly(bestX);
    orderTot = orderP + orderQ;
    % calculate adjusted value of the coefficient of determination
    glbRsqr = 1 - (1 - glbRsqr)*(n-1)/(n-orderTot-1);
    if bestR2 < glbRsqr
      bestR2 = glbRsqr;
      bestP = orderP;
      bestQ = orderQ;
      bestQSPcoeff = QSPcoeff;
      zBestX = bestX;
      bestYCalc = yCalc;
    end
  end
end
fprintf("Best orderP = %d\n", bestP);
fprintf("Best orderQ = %d\n", bestQ);
fprintf("Adjusted Rsqr = %f\n", bestR2);
fprintf("Quantum Shammas Pade Polynomial Powers\n");
zBestX
fprintf("Quantum Shammas Pade Polynomial Coefficients\n");
QSPcoeff = bestQSPcoeff'

fprintf("\nShammas Pade polynomial fit\n");
[LbP,UbP] = makeLimits(bestP, 0.5, 0.9, 0.1);
[LbQ,UbQ] = makeLimits(bestQ, 0.5, 0.9, 0.1);
ShamPolyPwrs = [LbP+(UbP - LbP)/2 LbQ+(UbQ - LbQ)/2];
[c2,r2,yPoly2] = ShammasPadefit(xData,yData,LbP+(UbP - LbP)/2,LbQ+(UbQ - LbQ)/2);
% calculate adjusted value of the coefficient of determination
orderTot = bestP + bestQ;
r2 = 1 - (1 - r2)*(n-1)/(n-orderTot-1);
fprintf("Adjusted Rsqr = %f\n", r2);

fprintf("\nRegular Pade polynomial fit\n");
[c,r,yPoly] = padefit(xData,yData,bestP,bestQ);
% calculate adjusted value of the coefficient of determination
orderTot = bestP + bestQ;
r = 1 - (1 - r)*(n-1)/(n-orderTot-1);
fprintf("Adjusted Rsqr = %f\n", r);
figure(1)
plot(xData,yData,xData,bestYCalc,xData,yPoly,xData,yPoly2);
title(sEqn)
xlabel("X")
ylabel("Y");
grid;
ax = gca;
exportgraphics(ax,gFile);

QSPpwr = zBestX;
Coeff = c';
Coeff2 = c2';
T = array2table(QSPpwr);
writetable(T,xlFile,"Sheet","Sheet1","Range","A1");
T = array2table(QSPcoeff);
writetable(T,xlFile,"Sheet","Sheet1","Range","A4");
T = array2table(ShamPolyPwrs);
writetable(T,xlFile,"Sheet","Sheet1","Range","A7");
T = array2table(Coeff2);
writetable(T,xlFile,"Sheet","Sheet1","Range","A10");
T = array2table(QSPcoeff);
writetable(T,xlFile,"Sheet","Sheet1","Range","A13");

T = array2table(Coeff);
writetable(T,xlFile,"Sheet","Sheet1","Range","A16");
r_sqr = [bestR2 r2 r];
T = array2table(r_sqr);
writetable(T,xlFile,"Sheet","Sheet1","Range","A19");
padeOrderP = bestP;
padeOrderQ = bestQ;
orderP = [padeOrderP];
T = array2table(orderP);
writetable(T,xlFile,"Sheet","Sheet1","Range","E19");
orderQ = [padeOrderQ];
T = array2table(orderQ);
writetable(T,xlFile,"Sheet","Sheet1","Range","F19");

format short
diary off

function [Lb,Ub] = makeLimits(order, minPwr, maxPwr, gap)
  Lb = zeros(1,order);
  Ub = zeros(1,order);
  delta = maxPwr - minPwr;
  Lb(1) = minPwr;
  Ub(1) = maxPwr;
  for i=2:order
    Lb(i) = Ub(i-1) + gap;
    Ub(i) = Lb(i) + delta;
  end
end


